# Ubuntu Developer Summit web application
# Copyright (C) 2008, 2009, 2010 Canonical Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import urlparse

from django import forms
from django.forms.util import ValidationError
from django.utils.translation import ugettext_lazy as _

from summit.brainstorm.models import Idea
from summit.brainstorm.widgets import BrainstormURLWidget

__all__ = (
    'BrainstormURLField',
)


class BrainstormURLField(forms.URLField):
    # FIXME this is a little too magic
    # we should have a field that simply validates a brainstorm url
    # and a different field that validates an entire brainstorm idea ref

    default_error_messages = {
        'not_brainstorm': _(u'This URL does not appear to be for Brainstorm'),
    }

    valid_schemes = ("http", "https")
    valid_netlocs = ("brainstorm.ubuntu.com", )
    path_re = re.compile(r'^/idea/(\d+)/')

    widget = BrainstormURLWidget

    def clean(self, value):
        url = super(BrainstormURLField, self).clean(value)

        info = urlparse.urlsplit(url)
        if info[0] not in self.valid_schemes:
            raise ValidationError(self.error_messages['not_brainstorm'])
        if info[1] not in self.valid_netlocs:
            raise ValidationError(self.error_messages['not_brainstorm'])

        match = self.path_re.match(info[2])
        if not match:
            raise ValidationError(self.error_messages['not_brainstorm'])

        idea_id = match.group(1)

        try:
            idea = Idea.objects.get(pk=idea_id)
        except Idea.DoesNotExist:
            import urllib2
            from BeautifulSoup import BeautifulSoup
            try:
                u = urllib2.urlopen(url)
                soup = BeautifulSoup(u)
            except urllib2.HTTPError:
                raise ValidationError(self.error_messages['not_brainstorm'])

            try:
                title = soup('h1')[0].string
                title.index("\nIdea\n")
                title = title[title.index(":")+1:].strip()
            except (IndexError, ValueError):
                raise ValidationError(self.error_messages['not_brainstorm'])

            idea = Idea.objects.create(id=idea_id, url=url, title=title)

        return idea
